<?php

namespace Drupal\oidc\EventSubscriber;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Routing\RouteBuildEvent;
use Drupal\Core\Routing\RoutingEvents;
use Drupal\oidc\OpenidConnectSessionInterface;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
 * Event subscriber for the route events.
 */
class RouteEventSubscriber implements EventSubscriberInterface {

  /**
   * General configuration.
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  protected $generalConfig;

  /**
   * Class constructor.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The configuration factory service.
   */
  public function __construct(ConfigFactoryInterface $config_factory) {
    $this->generalConfig = $config_factory->get('oidc.general');
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    $events[RoutingEvents::ALTER][] = 'onAlter';

    return $events;
  }

  /**
   * Alters the built routes.
   *
   * @param \Drupal\Core\Routing\RouteBuildEvent $event
   *   The route build event.
   */
  public function onAlter(RouteBuildEvent $event) {
    $collection = $event->getRouteCollection();

    if ($this->generalConfig->get('disable_user_routes')) {
      // Make the user register and pass routes inaccessible
      $collection->get('user.register')
        ->addRequirements([
          '_access' => 'FALSE',
        ]);

      $collection->get('user.pass')
        ->addRequirements([
          '_access' => 'FALSE',
        ]);
    }

    // Change the title callback for some user routes.
    $collection->get('entity.user.canonical')
      ->setDefault('_title_callback', 'oidc_user_title');

    $collection->get('entity.user.edit_form')
      ->setDefault('_title_callback', 'oidc_user_title');
  }

}
